'use strict';

var _amqp = require('amqp');

var _amqp2 = _interopRequireDefault(_amqp);

var _public = require('./public.js');

var _public2 = _interopRequireDefault(_public);

var _subscribe = require('./subscribe.js');

var _subscribe2 = _interopRequireDefault(_subscribe);

var _download = require('../utils/download.js');

var _download2 = _interopRequireDefault(_download);

var _redis = require('../utils/redis.js');

var _redis2 = _interopRequireDefault(_redis);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const redis = _redis2.default.getCli();
const WechatMsgTypeToCrmType = {
  text: 'TEXT',
  image: 'IMAGE',
  voice: 'AUDIO',
  ext: 'EXTENSION'
};

class Message {
  constructor(config, wechatapi) {
    this.exchange = null;
    this.config = config;
    this.rabbit = _amqp2.default.createConnection(this.config.mq.rabbitmq);
    this.wechatapi = wechatapi;
    this.stat = 1;
    this.eventEmitter = new _events2.default.EventEmitter();
  }
  start() {
    return new Promise((resolve, reject) => {
      console.log("start........");
      this.rabbit.on('ready', err => {
        console.log('rabbit ready.................................................');
        //接收mq
        this.subscribe();
        //推送mq
        this.initPushExchange();
        //连接成功
        resolve();
      });
      this.rabbit.on('error', err => {
        console.log(err);
      });
    });
  }
  subscribe() {
    this.rabbit.queue(this.config.mq.rabbitqueue, {
      durable: true,
      noDeclare: true
    }, q => {
      q.bind(this.config.mq.routing);
      q.subscribe(message => {
        //todo deal subscribed message
        let subscribe = new _subscribe2.default(this.wechatapi, this.config);
        subscribe.subscribe(message).then(() => {
          console.log("推送成功");
        }).catch(err => {
          console.log(err);
        });
      });
    });
  }

  initPushExchange() {
    try {
      this.exchange = this.rabbit.exchange(this.config.mq.exchange, {
        passive: true,
        durable: true,
        noDeclare: true,
        type: "topic"
      }, exc => {
        if (this.stat) {
          this.eventEmitter.on("publishCrmMsg", (message, callback) => {
            this.exchange.publish("#", message, {
              contentType: "application/json"
            }, function (err) {
              //todo log error
              console.log("publishCrmMsg..........");
            });
          });
        }
      });
    } catch (error) {
      //todo log error
      console.log(error);
    }
  }

  publishCrmMsg(info) {
    return new Promise((resolve, reject) => {
      const publicObj = new _public2.default(this.config, this.wechatapi);
      publicObj.buildMsgByInfo(info).then(msg => {
        console.log("publish msg" + JSON.stringify(msg));
        this.eventEmitter.emit("publishCrmMsg", msg);
        resolve(msg);
      }).catch(function (err) {
        reject(err);
      });
    });
  }
}

module.exports = Message;