'use strict';
import amqp from 'amqp';
import Public from './public.js';
import Subscribe from './subscribe.js';
import download from '../utils/download.js';
import redisUtil from '../utils/redis.js';
import events from 'events';

const redis = redisUtil.getCli();
const WechatMsgTypeToCrmType = {
  text: 'TEXT',
  image: 'IMAGE',
  voice: 'AUDIO',
  ext: 'EXTENSION'
};

class Message{
  constructor(config,wechatapi){
    this.exchange = null;
    this.config = config;
    this.rabbit = amqp.createConnection(this.config.mq.rabbitmq);
    this.wechatapi = wechatapi;
    this.stat = 1;
    this.eventEmitter = new events.EventEmitter();
  }
  start(){
    return new Promise((resolve,reject)=>{
      console.log("start........");
      this.rabbit.on('ready',(err)=>{
        console.log('rabbit ready.................................................');
        //接收mq
        this.subscribe();
        //推送mq
        this.initPushExchange();
        //连接成功
        resolve();
      });
      this.rabbit.on('error',(err)=>{
        console.log(err);
      });
    })
  }
  subscribe(){
    this.rabbit.queue(this.config.mq.rabbitqueue, {
      durable: true,
      noDeclare: true
    },(q)=>{
      q.bind(this.config.mq.routing);
      q.subscribe((message) =>{
        //todo deal subscribed message
        let subscribe = new Subscribe(this.wechatapi,this.config);
        subscribe.subscribe(message).then(()=>{
          console.log("推送成功");
        }).catch((err)=>{
          console.log(err);
        });
      });
    });
  }

  initPushExchange(){
    try{
      this.exchange = this.rabbit.exchange(this.config.mq.exchange, {
          passive: true,
          durable: true,
          noDeclare: true,
          type: "topic"
        },(exc)=>{
          if(this.stat){
            this.eventEmitter.on("publishCrmMsg",(message,callback)=>{
              this.exchange.publish("#", message, {
                contentType: "application/json"
              }, function(err) {
                //todo log error
                console.log("publishCrmMsg..........");
              })
            })
          }
        })
    }catch(error){
      //todo log error
      console.log(error)
    }
  }

  publishCrmMsg(info){
    return new Promise((resolve,reject)=>{
      const publicObj = new Public(this.config,this.wechatapi);
      publicObj.buildMsgByInfo(info).then((msg)=>{
        console.log("publish msg" + JSON.stringify(msg));
        this.eventEmitter.emit("publishCrmMsg",msg);
        resolve(msg);
      }).catch(function(err){
        reject(err);
      })
    })
  }
}





module.exports = Message

